@@ -15,6 +15,11 @@
 import sys, os, re, shutil
 import zc.buildout
 import zc.recipe.egg
+import logging
+
+join = os.path.join
+
+curdir = os.path.dirname(__file__)
 
 class Recipe:
 
@@ -33,7 +38,6 @@ class Recipe:
 
     _ws_locations = None
 
-    @property
     def ws_locations(self):
         if self._ws_locations is None:
             self._ws_locations = [d.location for d in self.zodb_ws]
@@ -44,6 +48,7 @@ class Recipe:
                 assert os.path.isdir(software_home), invalid_z2_location_msg
                 self._ws_locations.append(software_home)
         return self._ws_locations
+    ws_locations = property(ws_locations)
 
     def install(self):
         options = self.options
@@ -72,11 +77,11 @@ class Recipe:
         # this was taken from mkzeoinstance.py
         from ZEO.mkzeoinst import ZEOInstanceBuilder
         
-        zodb3_home = os.path.dirname(os.path.dirname(ZEO.__file__))
+        self.zodb3_home = os.path.dirname(os.path.dirname(ZEO.__file__))
         params = {
             "package": "zeo",
             "PACKAGE": "ZEO",
-            "zodb3_home": zodb3_home,
+            "zodb3_home": self.zodb3_home,
             "instance_home": location,
             "port": 8100, # will be overwritten later
             "python": options['executable'],
@@ -161,12 +166,22 @@ class Recipe:
                 os.makedirs(socket_dir)
 
             z_log_name = os.path.sep.join(('var', 'log', self.name + '.log',))
-            z_log_name = options.get('zeo-log', z_log_name)
-            z_log = os.path.join(base_dir, z_log_name)
-            z_log_dir = os.path.dirname(z_log)
+            zeo_log_custom = options.get('zeo-log-custom', None)
+            
+            # if zeo-log is given, we use it to set the runner
+            # logfile value in any case
+            z_log_filename = options.get('zeo-log', z_log_name)
+            z_log_filename = os.path.join(base_dir, z_log_filename)
+            z_log_dir = os.path.dirname(z_log_filename)
             if not os.path.exists(z_log_dir):
                 os.makedirs(z_log_dir)
 
+            # zeo-log-custom superseeds zeo-log
+            if zeo_log_custom is None:
+                z_log = z_log_file % {'filename': z_log_filename}
+            else:
+                z_log = zeo_log_custom
+
             file_storage = os.path.sep.join(('var', 'filestorage', 'Data.fs',))
             file_storage = options.get('file-storage', file_storage)
             file_storage = os.path.join(base_dir, file_storage)
@@ -197,6 +212,7 @@ class Recipe:
                 invalidation_queue_size = invalidation_queue_size,
                 socket_name = socket_name,
                 z_log = z_log,
+                z_log_filename = z_log_filename,
                 storage = storage,
                 zeo_address = zeo_address,
                 pid_file = pid_file,
@@ -224,9 +240,9 @@ class Recipe:
         options = self.options
         location = options['location']
 
-        zeo_conf = options.get('zeo-conf', None)
-        if zeo_conf is None:
-            zeo_conf = os.path.join(location, 'etc', 'zeo.conf')
+        self.zeo_conf = options.get('zeo-conf', None)
+        if self.zeo_conf is None:
+            self.zeo_conf = os.path.join(location, 'etc', 'zeo.conf')
 
         _, ws = self.egg.working_set(['plone.recipe.zope2zeoserver'])
 
@@ -240,7 +256,7 @@ class Recipe:
             initialization = initialization,
             arguments = ('\n        ["-C", %r]'
                          '\n        + sys.argv[1:]'
-                         % zeo_conf
+                         % self.zeo_conf
                          ),
             )
         # zeopack.py
@@ -265,6 +281,62 @@ class Recipe:
                 arguments='host, port',
                 )
 
+        
+        if sys.platform == 'win32':
+            self.install_win32_scripts()
+
+    def install_win32_scripts(self):
+            location = self.options['location']
+
+            # zeoservice.py
+            zeo_service = open(join(curdir, 'zeoservice.py')).read()
+            
+            zope2_location = self.options.get('zope2-location', None)
+            if zope2_location is None:
+                zope2_location = join(location, 'parts', 'zope2') 
+            
+            software_home = join(zope2_location, 'lib', 'python')
+            
+            arguments = {'PYTHON': self.options['executable'],
+                         'SOFTWARE_HOME': software_home,
+                         'zodb3_home': self.zodb3_home,
+                         'ZOPE_HOME': zope2_location,
+                         'INSTANCE_HOME': location,
+                         'PACKAGE': 'zeo'}
+            
+            zeo_file = os.path.join(self.options['bin-directory'], 'zeoservice.py')
+            self._write_file(zeo_file, zeo_service % arguments)
+            
+            initialization = """
+            import os; os.environ['PYTHONPATH'] = %r
+            """.strip() % os.path.pathsep.join(self.ws_locations)
+
+            zc.buildout.easy_install.scripts(
+                [('zeoservice', 'zeoservice', 'main')],
+                self.zodb_ws, self.options['executable'], self.options['bin-directory'],
+                extra_paths = self.ws_locations,
+                initialization = initialization,
+                )
+
+
+            # runzeo.bat
+            runzeo = open(join(curdir, 'runzeo.bat')).read()
+            self._write_file(os.path.join(self.options['bin-directory'], 
+                             'runzeo.bat'), runzeo % arguments)
+
+
+    def _write_file(self, path, content):
+        logger = logging.getLogger('zc.buildout.easy_install')
+        f = open(path, 'w')
+        try:
+            f.write(content)
+        finally:
+            f.close()
+        logger.debug('Wrote file %s' % path)
+        os.chmod(path, 0755)
+        logger.warning('Changed mode for %s to 755' % path)
+    
+
 invalid_z2_location_msg = """
 'zope2-location' doesn't point to an actual Zope2 SOFTWARE_HOME. Check this
 setting and, eventually, make sure this buildout part is being run after the
@@ -294,6 +366,13 @@ blob_storage_template = """
 </blobstorage>
 """.strip()
 
+z_log_file = """\
+     <logfile>
+      path %(filename)s
+      format %%(message)s
+    </logfile>
+""".strip()
+
 # The template used to build zeo.conf
 zeo_conf_template="""\
 %%define INSTANCE %(instance_home)s
@@ -309,10 +388,7 @@ zeo_conf_template="""\
 
 <eventlog>
   level info
-  <logfile>
-    path %(z_log)s
-    format %%(message)s
-  </logfile>
+  %(z_log)s
 </eventlog>
 
 <runner>
@@ -328,7 +404,7 @@ zeo_conf_template="""\
 
   # This logfile should match the one in the zeo.conf file.
   # It is used by zdctl's logtail command, zdrun/zdctl doesn't write it.
-  logfile %(z_log)s
+  logfile %(z_log_filename)s
 </runner>
 
 %(zeo_conf_additional)s
